import React from 'react';
import * as H from 'history';
import { Mode } from './config';
import { SwitchTab, RoughLocation } from '.';
export interface RouteConfig {
    /** 子路由 */
    children?: RouteConfig[];
    /** 子路由 */
    routes?: RouteConfig[];
    /** 在菜单中隐藏子节点 */
    hideChildrenInMenu?: boolean;
    /** 在菜单中隐藏自己和子节点 */
    hideInMenu?: boolean;
    /** 菜单的名字 */
    name?: string;
    /** 路径 */
    path?: string;
    /** 配置该路由标签页紧跟指定的某个路由 */
    follow?: string;
    component?: React.ComponentType<{
        location: H.Location;
    }>;
}
export interface RenderRoute extends Omit<RouteConfig, 'name'> {
    renderKey: string;
    /** Mode.Dynamic 会计算路由参数的 hash 值 */
    hash?: string;
    name?: React.ReactNode;
}
export interface SetTabNamePayload {
    path: string;
    name?: string;
    params: any;
    location: RoughLocation;
}
export declare type SetTabNameFn = (payload: SetTabNamePayload) => React.ReactNode | void;
export interface ActionType {
    reloadTab: (path?: string) => void;
    /** 如果已经打开的标签页会触发 callback ，如果 force 为 true ，总会调用 callback */
    goBackTab: (path?: string, callback?: () => void, force?: boolean) => void;
    /** 关闭后自动切换到附近的标签页，如果是最后一个标签页不可删除 */
    closeTab: (path?: string, callback?: () => void, force?: boolean) => void;
    closeAndGoBackTab: (path?: string, callback?: () => void, force?: boolean) => void;
}
export interface UseSwitchTabsOptions {
    mode?: Mode;
    /** tabs 持久化 */
    persistent?: {
        /** 是否强制渲染，参考 [Tabs.TabPane.forceRender](https://ant.design/components/tabs-cn/#Tabs.TabPane) */
        force?: boolean;
    } | boolean;
    children: JSX.Element;
    originalRoutes: RouteConfig[];
    location: H.Location;
    history: Pick<H.History, 'push'>;
    actionRef?: React.MutableRefObject<ActionType | undefined> | ((actionRef: ActionType) => void);
    /** Mode.Dynamic 时可用  */
    setTabName?: SetTabNameFn;
}
declare function useSwitchTabs(options: UseSwitchTabsOptions): {
    tabs: SwitchTab[];
    activeKey: string;
    handleSwitch: (keyToSwitch: string, callback?: () => void, force?: boolean) => void;
    handleRemove: (removeKey: string, nextTabKey?: string, callback?: () => void, force?: boolean) => void;
    handleRemoveOthers: (currentKey: string, callback?: () => void) => void;
    handRemoveRightTabs: (currentKey: string, callback?: () => void) => void;
};
export default useSwitchTabs;
