"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isRouteTab = isRouteTab;
exports.getParams = getParams;
exports.getRenderRoute = getRenderRoute;
exports.withSwitchTab = withSwitchTab;
exports.getRenderRouteKey = getRenderRouteKey;
exports.routePagePropsAreEqual = void 0;

var _react = _interopRequireDefault(require("react"));

var _find2 = _interopRequireDefault(require("lodash/find"));

var _isEqual2 = _interopRequireDefault(require("lodash/isEqual"));

var _isEmpty2 = _interopRequireDefault(require("lodash/isEmpty"));

var _hashString = _interopRequireDefault(require("hash-string"));

var _pathToRegexp = require("@qixian.cs/path-to-regexp");

var _config = require("./config");

var _excluded = ["children", "computedMatch", "history", "location", "match", "route", "staticContext"],
    _excluded2 = ["children", "computedMatch", "history", "location", "match", "route", "staticContext"];

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function isRouteTab(location, originalRoutes) {
  function isInMenus(menuData) {
    var targetMenuItem = (0, _find2.default)(menuData, function (item) {
      return (0, _pathToRegexp.pathToRegexp)("".concat(item.path, "(.*)")).test(location.pathname);
    });
    return !!targetMenuItem;
  }

  return isInMenus(originalRoutes);
}

var pathnameMapCache = {};
/**
 * 解析 `RenderRoute`，核心是算出合适的 `renderKey`
 *
 * @param location
 * @param originalRoutes 原始路由数据，未经过滤处理
 */

function getOriginalRenderRoute(location, originalRoutes) {
  var pathname = location.pathname;

  if (pathnameMapCache[pathname]) {
    return pathnameMapCache[pathname];
  }

  function getMetadata(menuData, parent) {
    var result;
    /** 根据前缀匹配菜单项，因此，`BasicLayout` 下的 **一级路由** 只要配置了 `name` 属性，总能找到一个 `path` 和 `name` 的组合 */

    var targetRoute = (0, _find2.default)(menuData, function (item) {
      return (0, _pathToRegexp.pathToRegexp)("".concat(item.path, "(.*)")).test(pathname) && !!item.name;
    });
    /** 如果为 **一级路由** 直接写入 `result` ，否则父级没有 `component` 时才能写入 `result` */

    if (!parent && targetRoute || parent && !parent.component && targetRoute) {
      result = _objectSpread({
        renderKey: targetRoute.path
      }, targetRoute);
    }
    /** 如果父级配置了 `hideChildrenInMenu` ，子级配置了 `name` 则重写 `result` */


    if ((parent === null || parent === void 0 ? void 0 : parent.hideChildrenInMenu) && targetRoute) {
      result = _objectSpread({
        renderKey: parent.path
      }, targetRoute);
    }
    /** 递归设置 `renderKey` */


    if (Array.isArray(targetRoute === null || targetRoute === void 0 ? void 0 : targetRoute.children) && (targetRoute === null || targetRoute === void 0 ? void 0 : targetRoute.children.length)) {
      result = getMetadata(targetRoute.children, targetRoute) || result;
    }

    if (Array.isArray(targetRoute === null || targetRoute === void 0 ? void 0 : targetRoute.routes) && (targetRoute === null || targetRoute === void 0 ? void 0 : targetRoute.routes.length)) {
      result = getMetadata(targetRoute.routes, targetRoute) || result;
    }

    return result;
  }

  var result = getMetadata(originalRoutes, null);
  /** 在存在页面子路由，如果不赋值 pathname，path 始终指向父路由 */

  pathnameMapCache[pathname] = _objectSpread(_objectSpread({}, result), {}, {
    path: pathname
  });
  return pathnameMapCache[pathname];
}
/**
 * 解析路由定义中参数
 *
 * 如： `path = /user/:id` ，`pathname = /user/48` ，可解析得到 `{ id: "48" }`
 *
 * @param path 路由定义
 * @param pathname 当前的页面路由
 */


function getParams(path, pathname) {
  var match = (0, _pathToRegexp.match)(path);
  var result = match(pathname);
  return result.params;
}
/**
 * 获取要激活的标签页信息
 *
 * @param options 其中，`location` 必须是 `react-router` 注入的 `location`，否则部署到非根目录功能时功能异常
 * @returns
 */


function getRenderRoute(options) {
  var location = options.location,
      mode = options.mode,
      originalRoutes = options.originalRoutes,
      setTabName = options.setTabName;
  var renderRoute = getOriginalRenderRoute(location, originalRoutes);

  if (!renderRoute || mode === _config.Mode.Route) {
    return renderRoute;
  } // 以下为 **路径** 模式的处理逻辑：
  // 核心是根据路由中所带的参数算出参数的哈希值，之后可将其与算出的 `renderKey` 拼成一个标签页的唯一 id
  // 这样，不同的参数就能得到不同的标签页了


  var params = getParams(renderRoute.renderKey, location.pathname);
  var search = location.search,
      _location$state = location.state,
      state = _location$state === void 0 ? {} : _location$state;
  var hashString = '';

  if (!(0, _isEmpty2.default)(params) || !(0, _isEmpty2.default)(search) || !(0, _isEmpty2.default)(state)) {
    hashString = (0, _hashString.default)(JSON.stringify(_objectSpread(_objectSpread({}, params), {}, {
      /**
       * 如果在 router.push 的时候设置 query ，可能导致查询参数为 number 类型，在点击标签页标题的时候又会变为 string 类型
       * 导致了计算的 hash 值可能不唯一
       * 故统一转换为 string 类型
       */
      search: search
    }, state)));
  }

  return _objectSpread(_objectSpread({}, renderRoute), {}, {
    hash: hashString,
    name: (setTabName === null || setTabName === void 0 ? void 0 : setTabName({
      path: renderRoute.path,
      name: renderRoute.name,
      params: params,
      location: location
    })) || renderRoute.name
  });
}

var routePagePropsAreEqual = function routePagePropsAreEqual(prevProps, nextProps) {
  var prevChildren = prevProps.children,
      prevComputedMatch = prevProps.computedMatch,
      prevHistory = prevProps.history,
      prevLocation = prevProps.location,
      prevMatch = prevProps.match,
      prevRoute = prevProps.route,
      prevStaticContext = prevProps.staticContext,
      prevRest = _objectWithoutProperties(prevProps, _excluded);

  var nextChildren = nextProps.children,
      nextComputedMatch = nextProps.computedMatch,
      nextHistory = nextProps.history,
      nextLocation = nextProps.location,
      nextMatch = nextProps.match,
      nextRoute = nextProps.route,
      nextStaticContext = nextProps.staticContext,
      nextRest = _objectWithoutProperties(nextProps, _excluded2);

  if (!(0, _isEqual2.default)(prevRest, nextRest)) {
    console.log("".concat(prevLocation.pathname, ": update by props")); // console.log(prevRest);
    // console.log(nextRest);

    return false;
  }

  var _ref = prevLocation || {},
      prevPathname = _ref.pathname,
      prevSearch = _ref.search,
      prevState = _ref.state;

  var _ref2 = nextLocation || {},
      nextPathname = _ref2.pathname,
      nextSearch = _ref2.search,
      nextState = _ref2.state;

  var isLocationChange = prevPathname !== nextPathname || prevSearch !== nextSearch || !(0, _isEqual2.default)(prevState, nextState);

  if (isLocationChange) {
    console.log("".concat(prevLocation.pathname, " -> ").concat(nextPathname, ": update by route or params")); // console.log({ prevPathname, prevSearch, prevState });
    // console.log({ nextPathname, nextSearch, nextState });

    return false;
  }

  console.log("without re-render: ".concat(prevPathname));
  return true;
};

exports.routePagePropsAreEqual = routePagePropsAreEqual;

function withSwitchTab(WrappedComponent) {
  var WithRoutePage = /*#__PURE__*/_react.default.memo(function (props) {
    // useConsole(props.location.pathname);
    return /*#__PURE__*/_react.default.createElement(WrappedComponent, _objectSpread({}, props));
  }, routePagePropsAreEqual);

  WithRoutePage.displayName = "WithRoutePage(".concat(getDisplayName(WrappedComponent), ")");
  return WithRoutePage;
}

function getDisplayName(WrappedComponent) {
  return WrappedComponent.displayName || WrappedComponent.name || 'Component';
}

function getRenderRouteKey(renderRoute, mode) {
  if (mode === _config.Mode.Dynamic && (renderRoute === null || renderRoute === void 0 ? void 0 : renderRoute.hash)) {
    return "".concat(renderRoute.renderKey, "-").concat(renderRoute.hash);
  }

  return renderRoute.renderKey;
}